name='altered.states'
version='0.8.6'
licence='ISC'
author='Jacob Oscarson'
author_email='jacob@plexical.com'
description = 'Monkey patching for Humans'
long_description = """
=================================
Python monkey-patching for Humans
=================================

*Altered States* tries to take the concept of "Python for Humans"
(http://python-for-humans.heroku.com/) to the world of monkey
patching.

Read more here:
https://github.com/Plexical/altered.states/blob/master/README.rst

Initial announcement:
http://www.plexical.com/blog/2012/03/06/altered-states/
"""
url = 'https://github.com/Plexical/altered.states'



"""
This isn't a test suite in the traditional sense (testing
correctness), rather a test suite to verify assumptions I make about
how Altered States should behave in a "realistic" environment.
"""
from __future__ import with_statement
import sys
import os
import subprocess

import pytest

from StringIO import StringIO

from altered import state, Expando

def test_capture_stdout():
    buf = StringIO()
    print('Foo')
    with(state(sys, stdout=buf)):
        print('Bar')
    assert buf.getvalue() == 'Bar' + os.linesep

# Bit of a pain to set up, but sometime:
# @state(os.environ, DJANGO_SETTINGS_MODULE='proj.settings')
# def test_django_case():
#     from proj import models
#     stuff = models.Model.objects.filter(... # you get the idea

def test_patch_sys_modules():
    with(state(sys.modules, fakey=Expando(foo='bar'))):
        import fakey
        assert fakey.foo == 'bar'
    with(pytest.raises(ImportError)):
        import fakey

@state(globals(), injected='foo')
def test_patch_module():
    assert injected == 'foo'

@state(vars(), injected='foo')
def test_patch_inplace():
    assert injected == 'foo'

def test_os_environ():
    with state(os.environ, ALTERED='states'):
        assert os.environ['ALTERED'] == 'states'

    assert 'ALTERED' not in os.environ

def test_decorator_os_environ():
    @state(os.environ, ALTERED='states')
    def check():
        return os.environ['ALTERED']

    assert check() == 'states'
    assert 'ALTERED' not in os.environ

@pytest.fixture
def w_getitem():
    class implements_getitem(object):
        def __getitem__(self, key):
            return 'foo'
    return implements_getitem()

def test_getitem_overridable(w_getitem):
    with state(w_getitem, any='changed'):
        assert w_getitem.any == 'changed'



# Copyright (C) 2007-2010 Michael Foord
# E-mail: michael AT voidspace DOT org DOT uk
# http://pypi.python.org/pypi/contextdecorator
#
# Copyright (c) 2007-2010, Michael Foord
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:

# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.

# Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Create objects that act as both context managers *and* as decorators, and behave
the same in both cases.

Context managers inheriting from ``ContextDecorator`` have to implement
``__enter__`` and ``__exit__`` as normal. ``__exit__`` retains its optional
exception handling even when used as a decorator.

Example::

   from contextlib import ContextDecorator

   class mycontext(ContextDecorator):
      def __enter__(self):
         print 'Starting'
         return self

      def __exit__(self, *exc):
         print 'Finishing'
         return False

   @mycontext()
   def function():
      print 'The bit in the middle'

   with mycontext():
      print 'The bit in the middle'

Existing context managers that already have a base class can be extended by
using ``ContextDecorator`` as a mixin class::

   from contextlib import ContextDecorator

   class mycontext(ContextBaseClass, ContextDecorator):
      def __enter__(self):
         return self

      def __exit__(self, *exc):
         return False

'''

import sys

try:
    from functools import wraps
except ImportError:
    # Python 2.4 compatibility
    def wraps(original):
        def inner(f):
            f.__name__ = original.__name__
            return f
        return inner

# horrible reraise code for compatibility
# with Python 2 & 3
if sys.version_info >= (3,0):
    exec ("""
def _reraise(cls, val, tb):
    raise val
""")
else:
    exec ("""
def _reraise(cls, val, tb):
    raise cls, val, tb
""")

try:
    next
except NameError:
    # Python 2.4 / 2.5
    def next(gen):
        return gen.next()

__all__ = ['__version__', 'ContextDecorator', 'contextmanager']

__version__ = '0.10.0'


_NO_EXCEPTION = (None, None, None)

class ContextDecorator(object):
    "A base class or mixin that enables context managers to work as decorators."

    def __call__(self, f):
        @wraps(f)
        def inner(*args, **kw):
            self.__enter__()

            exc = _NO_EXCEPTION
            try:
                result = f(*args, **kw)
            except Exception:
                exc = sys.exc_info()

            catch = self.__exit__(*exc)

            if not catch and exc is not _NO_EXCEPTION:
                _reraise(*exc)
            return result
        return inner



class GeneratorContextManager(ContextDecorator):
    """Helper for @contextmanager decorator."""

    def __init__(self, gen):
        self.gen = gen

    def __enter__(self):
        try:
            return next(self.gen)
        except StopIteration:
            raise RuntimeError("generator didn't yield")

    def __exit__(self, type, value, traceback):
        if type is None:
            try:
                next(self.gen)
            except StopIteration:
                return
            else:
                raise RuntimeError("generator didn't stop")
        else:
            if value is None:
                # Need to force instantiation so we can reliably
                # tell if we get the same exception back
                value = type()
            try:
                self.gen.throw(type, value, traceback)
                raise RuntimeError("generator didn't stop after throw()")
            except StopIteration:
                # Suppress the exception *unless* it's the same exception that
                # was passed to throw().  This prevents a StopIteration
                # raised inside the "with" statement from being suppressed
                exc = sys.exc_info()[1]
                return exc is not value
            except:
                # only re-raise if it's *not* the exception that was
                # passed to throw(), because __exit__() must not raise
                # an exception unless __exit__() itself failed.  But throw()
                # has to raise the exception to signal propagation, so this
                # fixes the impedance mismatch between the throw() protocol
                # and the __exit__() protocol.
                #
                if sys.exc_info()[1] is not value:
                    raise


def contextmanager(func):
    """@contextmanager decorator.

    Typical usage:

        @contextmanager
        def some_generator(<arguments>):
            <setup>
            try:
                yield <value>
            finally:
                <cleanup>

    This makes this:

        with some_generator(<arguments>) as <variable>:
            <body>

    equivalent to this:

        <setup>
        try:
            <variable> = <value>
            <body>
        finally:
            <cleanup>

    """
    @wraps(func)
    def helper(*args, **kwds):
        return GeneratorContextManager(func(*args, **kwds))
    return helper



from __future__ import with_statement

import os
import sys
import doctest
from StringIO import StringIO

import pytest

from altered import state, forget, alter, Expando

@pytest.fixture
def obj():
    return Expando(a=1)

@pytest.fixture
def dct():
    return {'a': 1}

@pytest.fixture
def objtest():
    def o_check(obj, a, b):
        assert obj.a == a
        assert obj.b == b
    return o_check

@pytest.fixture
def dcttest():
    def dct_check(dct, a, b):
        assert dct['a'] == a
        assert dct['b'] == b
    return dct_check

@pytest.fixture
def raisetest():
    def raise_check(b0rked):
        badness = Exception('Ops!')
        try:
            b0rked(badness)
        except Exception, e:
            assert e is badness
    return raise_check

def test_state_obj_extra(obj):
    with(state(obj, b=2, c=3)):
        assert obj.b == 2
        assert obj.c == 3
    assert not hasattr(obj, 'b')
    assert not hasattr(obj, 'c')

def test_state_obj_overwrite(obj):
    with(state(obj, a=2)):
        assert obj.a == 2
    assert obj.a == 1

def test_state_obj_forget(obj):
    with(state(obj, a=forget)):
        assert not hasattr(obj, 'a')
    assert obj.a == 1

def test_state_obj_raises(obj, raisetest):
    def b0rked(badness):
        with(state(obj, a=2)):
            raise badness

    raisetest(b0rked)
    assert obj.a == 1

def test_state_dict_extra(dct):
    with(state(dct, b=2, c=3)):
        assert dct == {'a':1, 'b':2, 'c':3}
    assert dct == {'a':1}

def test_state_dict_overwrite(dct):
    with(state(dct, a=2)):
        assert dct == {'a':2}
    assert dct == {'a':1}

def test_state_dict_forget(dct):
    with(state(dct, a=forget)):
        assert dct == {}
    assert dct == {'a':1}

def test_state_dct_raises(dct, raisetest):
    def b0rked(badness):
        with(state(dct, a=2)):
            raise badness

    raisetest(b0rked)
    assert dct['a'] == 1

def test_decorator_obj(obj, objtest):
    @state(obj, a=3, b=4)
    def check(o):
        objtest(o, 3, 4)
    check(obj)
    assert check.func_name == 'check'
    assert obj.a == 1
    assert not hasattr(obj, 'b')

def test_decorator_obj_raises(obj, raisetest):
    def b0rked(badness):
        @state(obj, a=2)
        def fn():
            raise badness
        fn()

    raisetest(b0rked)
    assert obj.a == 1

def test_decorator_dict(dct, dcttest):
    @state(dct, a=5, b=6)
    def check(d):
        dcttest(d, 5, 6)
    check(dct)
    assert check.func_name == 'check'
    assert dct['a'] == 1
    assert 'b' not in dct

def test_decorator_dct_raises(dct, raisetest):
    def b0rked(badness):
        @state(dct, a=2)
        def fn():
            raise badness
        fn()

    raisetest(b0rked)
    assert dct['a'] == 1

def test_alter_obj_extra(obj):
    restore = alter(obj, b=2, c=3)
    assert obj.b == 2
    assert obj.c == 3
    restore()
    assert not hasattr(obj, 'b')
    assert not hasattr(obj, 'c')

def test_alter_obj_overwrite(obj):
    restore = alter(obj, a=2)
    assert obj.a == 2
    restore()
    assert obj.a == 1

def test_alter_obj_forget(obj):
    restore = alter(obj, a=forget)
    assert not hasattr(obj, 'a')
    restore()
    assert obj.a == 1

def test_alter_dict_extra(dct):
    restore = alter(dct, b=2, c=3)
    assert dct == {'a': 1, 'b': 2, 'c': 3}
    restore()
    assert dct == {'a': 1}

# XXX todo 1, parametrize alt. new try with py.test's real doctest runner
def doctests(path):
    buf = StringIO()
    with(state(sys, stdout=buf)):
        doctest.testfile(path)
    res = buf.getvalue()
    if res != '':
        raise Exception(res)

@pytest.mark.skipif("sys.version_info[0:2] == (2,5)")
def test_readme():
    doctests('../README.rst')

@pytest.mark.skipif("sys.version_info[0:2] == (2,5)")
def test_sphinxindex():
    doctests('../docs/index.rst')

@pytest.mark.skipif("sys.version_info[0:2] == (2,5)")
def test_sphinx_examples():
    doctests('../docs/examples.rst')



from __future__ import with_statement

from functools import wraps

from altered.base import (dictget, dictset, dictdel, change,
                          restore, Expando, E, forget, dictlike)

try:
    from contextlib import ContextDecorator
except ImportError:
    from contextdecorator import ContextDecorator

def changers(obj):
    """
    Chooses suitable change operations for `obj`.
    """
    return (dictlike(obj) and (dictget, dictset, dictdel) or
            (getattr, setattr, delattr) )

class state(ContextDecorator):
    """
    This combined context manager and decorator is the main API to use
    Altered States.
    """

    def __init__(self, orig, **attrs):
        self.orig = orig
        self.attrs = attrs
        return super(state, self).__init__()

    def __enter__(self):
        self.getter, self.setter, self.deleter = changers(self.orig)
        self.diff = change(self.orig, self.getter,
                           self.setter, self.deleter, **self.attrs)
        return self

    def __exit__(self, *args, **kw):
        restore(self.orig, self.diff, self.getter, self.setter, self.deleter)
        return False

    def __call__(self, f):
        @wraps(f)
        def decorated(*args, **kwds):
            with self:
                return f(*args, **kwds)
        return decorated

def alter(obj, **changes):
    """
    Alternative API entry: a two step altering procedure. Same  as
    `state`, but returns a function that will restore the changes at a
    later point.
    """
    getter, setter, deleter = changers(obj)

    diff = change(obj, getter, setter, deleter, **changes)

    def restoration():
        restore(obj, diff, getter, setter, deleter)

    return restoration



from altered import Expando

def test_falsy_when_new():
    assert not Expando()

def test_falsy_on_change():
    exp = Expando(x='y')
    del exp.x
    assert not exp

def test_truthy_when_new():
    assert Expando(content='something') and True or False

def test_truthy_on_change():
    exp = Expando()
    exp.x = 'y'
    assert exp



#!/usr/bin/env python

from altered import meta

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name=meta.name,
    version=meta.version,
    description='Python monkey patching for humans.',
    long_description=(meta.long_description + '\n\n' +
                      open('CHANGES.rst').read()),
    packages=('altered',),
    url=meta.url,
    author=meta.author,
    author_email=meta.author_email,
    license='ISC',
    classifiers=(
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Operating System :: OS Independent',
    ),
)
